import sys


def stdout_write(msg: str):
    """
    :param msg:
    :return:
    """
    sys.stdout.write(msg)
    sys.stdout.flush()
    return True


def stderr_write(msg: str):
    sys.stderr.write(msg)
    sys.stderr.flush()
    return True



import atexit
import queue
import re
import sys
import os
import time
import copy
import traceback

import logging

from pathlib import Path
from queue import Queue, Empty
from threading import Lock, Thread
from concurrent_log_handler import ConcurrentRotatingFileHandler
from seryou_py_log import servyou_print, FileLock

os_name = os.name


class ColorHandler(logging.Handler):
    """
    根据日志严重级别，显示成五彩控制台日志。
    强烈建议使用pycharm的 monokai主题颜色，这样日志的颜色符合常规的交通信号灯颜色指示，色彩也非常饱和鲜艳。
    设置方式为 打开pycharm的settings -> Editor -> Color Scheme -> Console Font 选择monokai
    """

    terminator = '\r\n' if os_name == 'nt' else '\n'
    bule = 96 if os_name == 'nt' else 36
    yellow = 93 if os_name == 'nt' else 33

    def __init__(self, stream=None):
        """
        Initialize the handler.

        If stream is not specified, sys.stderr is used.
        """
        logging.Handler.__init__(self)
        if stream is None:
            stream = sys.stdout
        self.stream = stream
        self._display_method = 7 if os_name == 'posix' else 0

    def flush(self):
        """
        Flushes the stream.
        """
        self.acquire()
        try:
            if self.stream and hasattr(self.stream, "flush"):
                self.stream.flush()
        finally:
            self.release()

    def __build_color_msg_with_no_backgroud_color(self, record_level, record_copy: logging.LogRecord, ):
        complete_msg = self.format(record_copy)
        if record_level == 10:
            # msg_color = ('\033[0;32m%s\033[0m' % msg)  # 绿色
            # print(msg1)
            msg_color = f'\033[0;32m{complete_msg}\033[0m'  # 绿色
        elif record_level == 20:
            # msg_color = ('\033[%s;%sm%s\033[0m' % (self._display_method, self.bule, msg))  # 青蓝色 36    96
            msg_color = f'\033[0;36m{complete_msg}\033[0m'
        elif record_level == 30:
            # msg_color = ('\033[%s;%sm%s\033[0m' % (self._display_method, self.yellow, msg))
            msg_color = f'\033[0;33m{complete_msg}\033[0m'
        elif record_level == 40:
            # msg_color = ('\033[%s;35m%s\033[0m' % (self._display_method, msg))  # 紫红色
            msg_color = f'\033[0;31m{complete_msg}\033[0m'
        elif record_level == 50:
            # msg_color = ('\033[%s;31m%s\033[0m' % (self._display_method, msg))  # 血红色
            msg_color = f'\033[0;31m{complete_msg}\033[0m'
        else:
            msg_color = f'{complete_msg}'
        return msg_color

    @staticmethod
    def __spilt_msg(log_level, msg: str):
        split_text = '- 级别 -'
        if log_level == 10:
            split_text = '- DEBUG -'
        elif log_level == 20:
            split_text = '- INFO -'
        elif log_level == 30:
            split_text = '- WARNING -'
        elif log_level == 40:
            split_text = '- ERROR -'
        elif log_level == 50:
            split_text = '- CRITICAL -'
        msg_split = msg.split(split_text, maxsplit=1)
        return msg_split[0] + split_text, msg_split[-1]

    def __repr__(self):
        level = logging.getLevelName(self.level)
        name = getattr(self.stream, 'name', '')
        if name:
            name += ' '
        return '<%s %s(%s)>' % (self.__class__.__name__, name, level)

    def emit(self, record: logging.LogRecord):
        """
        Emit a record.

        If a formatter is specified, it is used to format the record.
        The record is then written to the stream with a trailing newline.  If
        exception information is present, it is formatted using
        traceback.print_exception and appended to the stream.  If the stream
        has an 'encoding' attribute, it is used to determine how to do the
        output to the stream.
        """
        # noinspection PyBroadException
        try:
            # very_nb_print(record)
            # record.message = record.getMessage()
            # effective_information_msg = record.getMessage()  # 不能用msg字段，例如有的包的日志格式化还有其他字段
            # record_copy = copy.copy(record)  # copy是因为，不要因为要屏幕彩色日志而影响例如文件日志 叮叮日志等其他handler的格式。
            # record_copy.for_segmentation_color = '彩色分段标志属性而已'
            # del record_copy.msg
            # assist_msg = self.format(record_copy)
            # print(f'**  {assist_msg}  ** ')
            stream = self.stream

            msg_color = self.__build_color_msg_with_no_backgroud_color(record.levelno, copy.copy(record))
            # stream.write(msg_color)
            # stream.write(self.terminator)
            # self.flush()
            stream.write(msg_color + self.terminator)
            # self.flush()
        except Exception as e:
            servyou_print(e)
            servyou_print(traceback.format_exc())
            # self.handleError(record)


class FileHandlerWithBufferWindwos(ConcurrentRotatingFileHandler):
    """
     解决了多进程下文件切片问题，但频繁操作文件锁，带来程序性能巨大下降。
    """
    file_handler_list = []
    # 只能在windwos运行正常，windwos是多进程每个进程的变量has_start_emit_all_file_handler是独立的。linux是共享的。
    has_start_emit_all_file_handler = False

    @classmethod
    def _emit_all_file_handler(cls):
        while True:
            for hr in cls.file_handler_list:
                hr.rollover_and_do_write()
            time.sleep(1)

    @classmethod
    def start_emit_all_file_handler(cls):
        Thread(target=cls._emit_all_file_handler, daemon=True).start()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.buffer_msgs_queue = Queue()
        # 如果程序属于立马就能结束的，需要在程序结束前执行这个钩子，防止不到最后一秒的日志没记录到。
        atexit.register(self._when_exit)
        self.file_handler_list.append(self)
        if not self.has_start_emit_all_file_handler:
            self.start_emit_all_file_handler()
            self.__class__.has_start_emit_all_file_handler = True

    def _when_exit(self):
        pass
        self.rollover_and_do_write()

    def emit(self, record):
        """
        emit已经在logger的handle方法中加了锁，所以这里的重置上次写入时间和清除buffer_msgs不需要加锁了。
        :param record:
        :return:
        """
        try:
            msg = self.format(record)
            self.buffer_msgs_queue.put(msg)
        except Exception:
            self.handleError(record)

    def rollover_and_do_write(self, ):
        # servyou_print(self.buffer_msgs_queue.qsize())
        self._rollover_and_do_write()

    def _rollover_and_do_write(self):
        buffer_msgs = ''
        while True:
            try:
                msg = self.buffer_msgs_queue.get(block=False)
                buffer_msgs += msg
            except Empty:
                break
        if buffer_msgs:
            try:
                self._do_lock()
                try:
                    if self.shouldRollover(None):
                        self.doRollover()
                except Exception as e:
                    self._console_log("Unable to do rollover: %s" % (e,), stack=True)
                # servyou_print(len(self._buffer_msgs))
                self.do_write(buffer_msgs)
            finally:
                self._do_unlock()


class FileHandlerWithBufferLinux(FileHandlerWithBufferWindwos):
    """
     解决了多进程下文件切片问题，但频繁操作文件锁，带来程序性能巨大下降。
    """
    file_handler_list = []
    has_start_emit_all_file_handler_process_id_set = set()
    __lock_for_rotate = Lock()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.buffer_msgs_queue = Queue()
        atexit.register(self._when_exit)
        self.file_handler_list.append(self)
        if os.getpid() not in self.has_start_emit_all_file_handler_process_id_set:
            self.start_emit_all_file_handler()
            self.__class__.has_start_emit_all_file_handler_process_id_set.add(os.getpid())

    def rollover_and_do_write(self, ):
        # servyou_print(self.buffer_msgs_queue.qsize())
        with self.__lock_for_rotate:
            self._rollover_and_do_write()


ConcurrentRotatingFileHandlerWithBufferInitiativeLinux = ConcurrentRotatingFileHandler


class DayRotatingFileHandler(logging.Handler):
    """
    这个多进程按时间切片安全的。
    官方的 TimedRotatingFileHandler 在多进程下疯狂报错，
    """
    file_handler_list = []
    has_start_emit_all_file_handler_process_id_set = set()
    __lock_for_rotate = Lock()

    @classmethod
    def _emit_all_file_handler(cls):
        while True:
            for hr in cls.file_handler_list:
                hr._write_to_file()
            #  每隔一秒钟批量写入一次
            time.sleep(1)

    @classmethod
    def _start_emit_all_file_handler(cls):
        Thread(target=cls._emit_all_file_handler, daemon=True).start()

    def __init__(self, file_name: str, file_path: str, back_count=10):
        super().__init__()
        self.file_name = file_name
        self.file_path = file_path
        self.backupCount = back_count
        self.extMatch = re.compile(r"^\d{4}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
        self.extMatch2 = re.compile(r"^\d{2}-\d{2}-\d{2}(\.\w+)?$", re.ASCII)
        self._last_delete_time = time.time()

        self.buffer_msgs_queue = queue.Queue()
        atexit.register(self._write_to_file)
        self.file_handler_list.append(self)
        if os.getpid() not in self.has_start_emit_all_file_handler_process_id_set:
            self._start_emit_all_file_handler()
            self.__class__.has_start_emit_all_file_handler_process_id_set.add(os.getpid())

    def emit(self, record: logging.LogRecord):
        """
        emit已经在logger的handle方法中加了锁，所以这里的重置上次写入时间和清除buffer_msgs不需要加锁了。
        :param record:
        :return:
        """
        # noinspection PyBroadException
        try:
            msg = self.format(record)
            self.buffer_msgs_queue.put(msg)
        except Exception:
            self.handleError(record)
        try:
            servyou_print(str(self.format(record)))
        except:
            pass

    def _write_to_file(self):
        buffer_msgs = ''
        while True:
            try:
                msg = self.buffer_msgs_queue.get(block=False)
                buffer_msgs += msg + '\n'
            except queue.Empty:
                break
        if buffer_msgs:
            with FileLock(self.file_path / Path(f'_delete_{self.file_name}.lock')):
                time_str = time.strftime('%Y-%m-%d')
                new_file_name = self.file_name + '.' + time_str
                path_obj = Path(self.file_path) / Path(new_file_name)
                path_obj.touch(exist_ok=True)
                with path_obj.open(mode='a') as f:
                    f.write(buffer_msgs)
                if time.time() - self._last_delete_time > 60:
                    self._find_and_delete_files()
                    self._last_delete_time = time.time()

    def _find_and_delete_files(self):
        """
        这一段命名不规范是复制原来的官方旧代码。
        Determine the files to delete when rolling over.

        More specific than the earlier method, which just used glob.glob().
        """
        dirName = self.file_path
        baseName = self.file_name
        fileNames = os.listdir(dirName)
        result = []
        prefix = baseName + "."
        plen = len(prefix)
        for fileName in fileNames:
            if fileName[:plen] == prefix:
                suffix = fileName[plen:]
                # print(fileName, prefix,suffix)
                if self.extMatch.match(suffix) or self.extMatch2.match(suffix):
                    result.append(os.path.join(dirName, fileName))
        if len(result) < self.backupCount:
            result = []
        else:
            result.sort()
            result = result[:len(result) - self.backupCount]
        for r in result:
            Path(r).unlink()



import os
import abc

# 根据os导包
if os.name == 'nt':
    import win32con
    import win32file
    import pywintypes

    LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
    # The default value
    LOCK_SH = 0
    LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
    _overlapped = pywintypes.OVERLAPPED()
else:
    import fcntl


class BaseLock(metaclass=abc.ABCMeta):
    def __init__(self, lock_file_path: str):
        self.file_path = open(lock_file_path, 'a')

    @abc.abstractmethod
    def __enter__(self):
        raise NotImplemented

    @abc.abstractmethod
    def __exit__(self, exc_type, exc_val, exc_tb):
        raise NotImplemented


class WindowsFileLock(BaseLock):
    """
    windows os file lock
    """

    def __enter__(self):
        # 文件描述符
        self.hfile = win32file._get_osfhandle(self.file_path.fileno())
        win32file.LockFileEx(self.hfile, LOCK_EX, 0, 0xffff0000, _overlapped)

    def __exit__(self, exc_type, exc_val, exc_tb):
        win32file.UnlockFileEx(self.hfile, 0, 0xffff0000, _overlapped)


class LinuxFileLock(BaseLock):
    """
        linux os file lock
    """

    def __enter__(self):
        fcntl.flock(self.file_path, fcntl.LOCK_EX)

    def __exit__(self, exc_type, exc_val, exc_tb):
        fcntl.flock(self.file_path, fcntl.LOCK_UN)


FileLock = WindowsFileLock if os.name == 'nt' else LinuxFileLock



# Copyright 2022-2027 by Servyou PubDesktop group . All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of Vinay Sajip
# not be used in advertising or publicity pertaining to distribution
# of the software without specific, written prior permission.
# VINAY SAJIP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
# VINAY SAJIP BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
# IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
# 该配置文件均为默认配置，会被具体实例参数覆盖

# TYPE:bool 控制台是否使用有彩的日志
DEFAULUT_USE_COLOR_HANDLER = True
# TYPE:bool 是否同时将日志记录到记log文件中
DEFAULT_ADD_MULTIPROCESSING_SAFE_ROATING_FILE = False
"""
LOG_FILE_HANDLER_TYPE
日志切片方式, 该配置必须依赖 DEFAULT_ADD_MULTIPROCESSING_SAFE_ROATING_FILE_HANDLER开启
值可以设置为：
1）为使用多进程安全按日志文件大小切割的文件日志,这是本人实现的批量写入日志，减少操作文件锁次数，测试10进程快速写入文件，win上性能比第5种提高了100倍，linux提升5倍
2）为多进程安全按天自动切割的文件日志，同一个文件，每天生成一个新的日志文件。日志文件名字后缀自动加上日期。
3）为不自动切割的单个文件的日志(不切割文件就不会出现所谓进程安不安全的问题) 
"""
LOG_FILE_HANDLER_TYPE = 1

# TYPE:int 单位是M,每个文件的切片大小，超过多少后就自动切割
LOG_FILE_SIZE = 100
# TYPE:int 对同一个日志文件，默认最多备份几个文件，超过就删除了。
LOG_FILE_BACKUP_COUNT = 3
# TYPE:int 默认日志级别
LOG_LEVEL_FILTER = 10

# TYPE:str 默认为dev环境 该env日志文件生成在项目根路径下
ENV = 'dev'
LOG_ENV_PATH = {
    'test': '/usr/local/logs/daqworker/',
    'prod': '/usr/local/logs/daqworker/'
}
# TYPE:str
LOG_NAME = 'servyou_log'

FORMATTER_TEMPLATE_DEMO = {
    1: '[日志时间%(asctime)s][日志名称%(name)s][文件%(filename)s][第%(lineno)d行][日志等级%(levelname)s]日志信息:%(message)s',
    2: '%(asctime)s - %(name)s - %(filename)s - %(funcName)s - %(lineno)d - %(levelname)s - %(message)s',
    3: '%(asctime)s - %(name)s - [ File "%(pathname)s", line %(lineno)d, in %(funcName)s ] - %(levelname)s - %(message)s',
    4: '%(asctime)s - %(name)s - "%(filename)s" - %(funcName)s - %(lineno)d - %(levelname)s - %(message)s -               File "%(pathname)s", line %(lineno)d ',
    5: '%(asctime)s - %(name)s - "%(pathname)s:%(lineno)d" - %(funcName)s - %(levelname)s - %(message)s',
    6: '%(name)s - %(asctime)-15s - %(filename)s - %(lineno)d - %(levelname)s: %(message)s',
    7: '%(asctime)s - %(name)s - "%(filename)s:%(lineno)d" - %(levelname)s - %(message)s',
    8: '[p%(process)d_t%(thread)d] %(asctime)s - %(name)s - "%(pathname)s:%(lineno)d" - %(funcName)s - %(levelname)s - %(message)s',
    9: '[p%(process)d_t%(thread)d] %(asctime)s - %(name)s - "%(filename)s:%(lineno)d" - %(levelname)s - %(message)s',
    10: '%(computer_ip) %(computer_name)-[p%(process)d_t%(thread)d] %(asctime)s - %(name)s - "%(filename)s:%(lineno)d" - %(levelname)s - %(message)s',
    11: '%(asctime)s - [process:%(process)d][%(levelname)s][%(filename)s][line:%(lineno)d]: %(message)s ',

}
# 日志模板 TYPE:str/int
# FORMATTER_STYLE_CODE = FORMATTER_TEMPLATE_DEMO[10]
FORMATTER_STYLE_CODE = 10



import copy
import os
import sys
import typing
import logging
import threading
import multiprocessing
from functools import lru_cache

from pathlib import Path

from seryou_py_log import log_config
from seryou_py_log.load_log_config import servyou_print, get_host_ip
from seryou_py_log.handlers import (ColorHandler, FileHandlerWithBufferWindwos, FileHandlerWithBufferLinux,
                                    ConcurrentRotatingFileHandler, DayRotatingFileHandler,
                                    ConcurrentRotatingFileHandlerWithBufferInitiativeLinux
                                    )

OS_NAME = os.name


class ServyouLog(object):
    """
    一个日志管理类，用于创建logger、添加handler
    """
    logger_name_list = []
    logger_list = []
    preset_name_level_map = dict()
    _unique_lock = threading.Lock()
    _unique = None

    def __init__(self, logger_name: typing.Union[str, None] = None):
        """
        :param logger_name
        """
        if logger_name in (None, '', 'root') and multiprocessing.process.current_process().name == 'MainProcess':
            logger_name = log_config.LOG_NAME
        self._logger_name = logger_name
        self.preset_name_level_map[logger_name] = 1
        self.logger = logging.getLogger(logger_name)

    def __new__(cls, *args, **kwargs):
        if ServyouLog._unique is None or (args and args[0] not in cls.preset_name_level_map):
            servyou_print(f"新建日志实例:{args[0]}")
            with ServyouLog._unique_lock:
                ServyouLog._unique = object.__new__(cls)
        return ServyouLog._unique

    def preset_log_level(self, log_level_int=20):
        """
        提前设置锁定日志级别，当之后再设置该命名空间日志的级别的时候，按照提前预设的级别，无视之后设定的级别。
        主要是针对动态初始化的日志，在生成日志之后再去设置日志级别不方便。
        :param log_level_int:
        :return:
        """
        self.preset_name_level_map[self._logger_name] = log_level_int

    def get_logger_and_add_handlers(self, log_level_int: int = None, *, is_add_stream_handler=True,
                                    do_not_use_color_handler=None, log_path=None,
                                    log_file_size: int = None,
                                    log_file_handler_type: int = None,
                                    formatter_template: typing.Union[int, logging.Formatter] = None):
        """
       :param log_level_int: 日志输出级别，设置为0 10 20 30 40 50，分别对应原生logging.DEBUG(10)，logging.INFO(20)，logging.WARNING(30)，logging.ERROR(40),logging.CRITICAL(50)级别
       :param is_add_stream_handler: 是否打印日志到控制台
       :param do_not_use_color_handler :是否禁止使用color彩色日志
       :param log_path: 设置存放日志的文件夹路径,如果不设置，则取默认配置
       :param log_file_size :日志大小，单位M，默认100M
       :param log_file_handler_type :这个值可以设置为1 2 3
              1）为使用多进程安全按日志文件大小切割的文件日志
              2）为多进程安全按天自动切割的文件日志，同一个文件，每天生成一个日志
              3）为不自动切割的单个文件的日志(不切割文件就不会出现所谓进程安不安全的问题)
       :param formatter_template :日志模板，如果为数字，则为nb_log_config.py字典formatter_dict的键对应的模板，
                                1为formatter_dict的详细模板，2为简要模板,5为最好模板。
                                如果为logging.Formatter对象，则直接使用用户传入的模板。
       """
        if log_level_int is None:
            log_level_int = log_config.LOG_LEVEL_FILTER
        if do_not_use_color_handler is None:
            do_not_use_color_handler = not log_config.DEFAULUT_USE_COLOR_HANDLER
        log_filename = self._logger_name
        if log_config.LOG_NAME:
            log_filename = log_config.LOG_NAM
        if log_file_size is None:
            log_file_size = log_config.LOG_FILE_SIZE
        if log_path is None:
            log_path = log_config.LOG_ENV_PATH.get(log_config.ENV, Path(sys.path[1]).as_posix())
        if formatter_template is None:
            formatter_template = log_config.FORMATTER_STYLE_CODE
        self._logger_level = log_level_int * 10 if log_level_int < 10 else log_level_int
        assert self._logger_level in [0, 10, 20, 30, 40, 50], ValueError("日志等级错误")
        self._is_add_stream_handler = is_add_stream_handler
        self._do_not_use_color_handler = do_not_use_color_handler
        self._log_path = log_path
        self._log_filename = log_filename
        self._log_file_size = log_file_size
        assert log_file_handler_type in (None, 1, 2, 3), ValueError("LOG_FILE_HANDLER_TYPE配置只可为 1 2 3")
        self._log_file_handler_type = log_file_handler_type or log_config.LOG_FILE_HANDLER_TYPE
        if isinstance(formatter_template, int):
            if formatter_template not in log_config.FORMATTER_TEMPLATE_DEMO.keys():
                raise ValueError("FORMATTER_STYLE_CODE日志模板样式配置必须在模板序列中选择")
            self._formatter = log_config.FORMATTER_TEMPLATE_DEMO[formatter_template]
        elif isinstance(formatter_template, str):
            self._formatter = formatter_template
        else:
            raise ValueError('设置的 formatter_template 不正确')
        self.logger.setLevel(self._logger_level)
        self.__add_handlers()
        return self.logger

    def look_over_all_handlers(self):
        """
        pring  current  all handlers
        :return:
        """
        servyou_print(f'当前logName:{self._logger_name},当前logHandlers是: {self.logger.handlers}')

    def remove_all_handlers(self):
        """
        clear current handlers
        :return:
        """
        for hd in self.logger.handlers:
            self.logger.removeHandler(hd)

    def remove_handler_by_handler_class(self, handler_class: type):
        """
        remove handlers by type
        :param handler_class:logging.StreamHandler,ColorHandler,MongoHandler,ConcurrentRotatingFileHandler,MongoHandler,CompatibleSMTPSSLHandler的一种
        :return:
        """
        assert handler_class in [logging.StreamHandler, ColorHandler, ConcurrentRotatingFileHandler], TypeError(
            '设置的handler类型不正确')
        for handler in self.logger.handlers:
            if isinstance(handler, handler_class):
                self.logger.removeHandler(handler)

    def __add_a_hanlder(self, handlerx: logging.Handler):
        """
        add noe log header
        :param handlerx:
        :return:
        """
        handlerx.setLevel(self._logger_level)
        c_computer_id, c_computer_name = get_host_ip()
        log_pattern = self._formatter.replace('%(computer_ip)', c_computer_id).replace('%(computer_name)',
                                                                                       c_computer_name)
        handlerx.setFormatter(logging.Formatter(log_pattern))
        self.logger.addHandler(handlerx)

    def _judge_logger_has_handler_type(self, handler_type: type):
        for hr in self.logger.handlers:
            if isinstance(hr, handler_type):
                return True

    def __add_handlers(self):
        """
        register log handlers
        :return:
        """
        #  不写日志文件， 默认控制台打印：
        if not log_config.DEFAULT_ADD_MULTIPROCESSING_SAFE_ROATING_FILE:
            self.__add_a_hanlder(logging.StreamHandler())
            return True
        # register添加控制台日志
        if not (self._judge_logger_has_handler_type(ColorHandler) or self._judge_logger_has_handler_type(
                logging.StreamHandler)) and self._is_add_stream_handler:
            handler = ColorHandler() if not self._do_not_use_color_handler else logging.StreamHandler()
            self.__add_a_hanlder(handler)

        # register 添加多进程安全切片的文件日志
        if not (self._judge_logger_has_handler_type(ConcurrentRotatingFileHandler) or
                self._judge_logger_has_handler_type(FileHandlerWithBufferWindwos) or
                self._judge_logger_has_handler_type(FileHandlerWithBufferLinux) or
                self._judge_logger_has_handler_type(ConcurrentRotatingFileHandlerWithBufferInitiativeLinux) or
                self._judge_logger_has_handler_type(DayRotatingFileHandler) or
                self._judge_logger_has_handler_type(logging.FileHandler) or
                self._judge_logger_has_handler_type(ConcurrentRotatingFileHandler)) and all(
            [self._log_path, self._log_filename]):
            if not os.path.exists(self._log_path):
                os.makedirs(self._log_path)
            log_file = os.path.join(self._log_path, self._log_filename)
            file_handler = None
            # 日志文件大小切割
            if self._log_file_handler_type == 1:
                if OS_NAME == 'nt':
                    # 在win下使用这个ConcurrentRotatingFileHandler可以解决多进程安全切片，但性能损失惨重。
                    # 10进程各自写入10万条记录到同一个文件消耗15分钟。比不切片写入速度降低100倍。
                    file_handler = FileHandlerWithBufferWindwos(log_file,
                                                                maxBytes=self._log_file_size * 1024 * 1024,
                                                                backupCount=log_config.LOG_FILE_BACKUP_COUNT,
                                                                encoding="utf-8")
                elif OS_NAME == 'posix':
                    # linux下可以使用ConcurrentRotatingFileHandler，进程安全的日志方式。
                    # 10进程各自写入10万条记录到同一个文件消耗100秒，还是比不切片写入速度降低10倍。因为每次检查切片大小和文件锁的原因。
                    file_handler = FileHandlerWithBufferLinux(log_file,
                                                              maxBytes=self._log_file_size * 1024 * 1024,
                                                              backupCount=log_config.LOG_FILE_BACKUP_COUNT,
                                                              encoding="utf-8")

            # 多进程安全按天自动切割的文件
            elif self._log_file_handler_type == 2:
                file_handler = DayRotatingFileHandler(self._log_filename, self._log_path,
                                                      back_count=log_config.LOG_FILE_BACKUP_COUNT)
            # 为不自动切割
            elif self._log_file_handler_type == 3:
                file_handler = logging.FileHandler(log_file, mode='a', encoding='utf-8')
            self.__add_a_hanlder(file_handler)


@lru_cache()  # LogManager 本身也支持无限实例化
def get_logger(name: typing.Union[str, None], *, log_level_int: int = None,
               is_add_stream_handler=True,
               do_not_use_color_handler=None, log_path=None,
               log_file_size: int = None,
               log_file_handler_type: int = None,
               formatter_template: typing.Union[int, logging.Formatter] = None) -> logging.Logger:
    locals_copy = copy.copy(locals())
    locals_copy.pop('name')
    return ServyouLog(name).get_logger_and_add_handlers(**locals_copy)



import time
import sys
import importlib
import socket
import multiprocessing
import traceback

from pathlib import Path
from shutil import copyfile

from seryou_py_log.pring_tools import stdout_write, stderr_write
from seryou_py_log import log_config


def is_main_process():
    """
    当前进程是否是主进程
    :return:
    """
    return multiprocessing.process.current_process().name == 'MainProcess'


def get_host_ip() -> tuple:
    """
    get machine name info
    :return:
    """
    ip = ''
    host_name = ''
    try:
        sc = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sc.connect(('8.8.8.8', 80))
        ip = sc.getsockname()[0]
        host_name = socket.gethostname()
        sc.close()
    except Exception:
        pass
    return ip, host_name


def exec_load_log_config(config_path: str = None):
    """
    加载日志模块配置
    :param config_path: log config file abs path
    :return:
    """
    try:
        default_module = importlib.import_module('log_config')
        if config_path:
            config_module = importlib.import_module(config_path)
        else:
            config_module = default_module
        importlib.reload(config_module)
        for var_name, var_value in config_module.__dict__.items():
            if var_name.isupper():
                setattr(default_module, var_name, var_value)
    except ModuleNotFoundError:
        auto_create_config_file_to_project_root_path()
        stdout_write(f'{time.strftime("%H:%M:%S")} \n \033[0m')
    return True


def auto_create_config_file_to_project_root_path():
    """
    配置文件加载失败后，生成默认配置文件2 Project Root Path
    :return:
    """
    if Path(sys.path[1]).as_posix() == Path(__file__).parent.absolute().as_posix():
        servyou_print('当前路径不可创建配置文件')
        return None
    copyfile(Path(__file__).parent / Path('log_config.py'), Path(sys.path[1]) / Path('log_config.py'))
    return True


def servyou_print(*args, sep=' ', end='\n', file=None):
    """
    复写print
    :param args:
    :param sep:
    :param end:
    :param file:
    :return:
    """
    args = (str(arg) for arg in args)
    if file == sys.stderr:
        # 标准错误 保持原始的红色错误方式，不希望转成蓝色
        stderr_write(sep.join(args))
    else:
        # 获取被调用函数在被调用时所处代码行数
        line = sys._getframe().f_back.f_lineno
        # 获取被调用函数所在模块文件名
        file_name = sys._getframe(1).f_code.co_filename
        stdout_write(
            f'\033[0;34m{time.strftime("%Y-%m-%d %H:%M:%S")} {file_name}:{line}   {sep.join(args)} {end} \033[0m')
    return True


exec_load_log_config()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="servyou-log",
    version="1.4",
    author="PubDesktopLsw",
    description='日志工具类',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    platforms=["all"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>2',
    install_requires=[
        'concurrent-log-handler == 0.9.11',
    ]
)
